// +-------------------------------- DISCLAIMER ---------------------------------+
// |                                                                             |
// | This application program is provided to you free of charge as an example.   |
// | Despite the considerable efforts of Euresys personnel to create a usable    |
// | example, you should not assume that this program is error-free or suitable  |
// | for any purpose whatsoever.                                                 |
// |                                                                             |
// | EURESYS does not give any representation, warranty or undertaking that this |
// | program is free of any defect or error or suitable for any purpose. EURESYS |
// | shall not be liable, in contract, in torts or otherwise, for any damages,   |
// | loss, costs, expenses or other claims for compensation, including those     |
// | asserted by third parties, arising out of or in connection with the use of  |
// | this program.                                                               |
// |                                                                             |
// +-----------------------------------------------------------------------------+
//
// ImageProcessing.cpp: implementation of the ImageProcessing class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ClinicalAnalysis.h"
#include "ImageProcessing.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

using namespace Euresys::Open_eVision_1_2;

ImageProcessing::ImageProcessing()
{

	m_ReferenceAngle = 0.0f;
	m_ReferencePositionX = 0.0f;
	m_ReferencePositionY = 0.0f;

	p_LineGauge = NULL;
	p_PointGauge = NULL;


}

ImageProcessing::~ImageProcessing()
{

}

 void ImageProcessing::Initialize()
{

	try
	{
	 
		/// Load the EasyGauge model file 
		m_WorldShape.Load("..\\Resources\\Model\\ClinicalAnalysis.GGE", true);
		p_LineGauge = (ELineGauge*)m_WorldShape.GetDaughter(0);
		p_PointGauge = (EPointGauge*)p_LineGauge->GetDaughter(0);

		/// Load the reference image 
		/// This reference image was used to create the flexible mask
		m_SrcImg.Load("..\\Resources\\Images\\ClinicalAnalysis_01.bmp");

		/// Evaluate the reference image offset and rotation
		m_WorldShape.Process(&m_SrcImg);
		m_ReferenceAngle = p_LineGauge->GetMeasuredLine().GetAngle();
		m_ReferencePositionX = p_LineGauge->GetMeasuredLine().GetCenterX() + p_PointGauge->GetMeasuredPoint().GetCenter().GetX();
		m_ReferencePositionY = p_LineGauge->GetMeasuredLine().GetCenterY() + p_PointGauge->GetMeasuredPoint().GetCenter().GetY();

		/// Load the corresponding flexible mask 
		m_MaskImg.Load("..\\Resources\\Mask\\FlexMask.bmp");

		/// Resize the temporary mask image
		m_MaskProcess.SetSize(&m_MaskImg);

		/// Attach an ROI to it
		m_MaskROI.Attach(&m_MaskProcess); 

		/// Empty the source image object
		m_SrcImg.SetSize(0, 0);


 	}
	catch(EException exc)
	{
		::MessageBox(NULL, exc.What().c_str(), "Error", MB_OK | MB_ICONERROR);
	}

	
}

void ImageProcessing::Inspect(LPCTSTR _mFilename)
{
	/// Temporay image for preprocessing
	EImageBW8 m_TempImg;
	
	try
	{
	
		/// Load the image for inspection
		m_SrcImg.Load(_mFilename);

		/// Resize the temporary image accordingly
		m_ImgProcess.SetSize(&m_SrcImg);

		/// Attach an ROI to it
		m_SrcROI.Attach(&m_ImgProcess);

		/// Evaluate the offset and the rotation for the current image 
		m_WorldShape.Process(&m_SrcImg, true);
		float m_Angle	= p_LineGauge->GetMeasuredLine().GetAngle() - m_ReferenceAngle;
		float m_X1		= p_LineGauge->GetMeasuredLine().GetCenterX() + p_PointGauge->GetMeasuredPoint().GetCenter().GetX();
		float m_Y1		= p_LineGauge->GetMeasuredLine().GetCenterY()  + p_PointGauge->GetMeasuredPoint().GetCenter().GetY();

		/// Register the mask according to the offset and the rotation of the current image
		EasyImage::ScaleRotate(&m_MaskImg, m_ReferencePositionX, m_ReferencePositionY, m_X1, m_Y1, 1.000f, 1.000f, -m_Angle, &m_MaskProcess, 8);

		for (int i = 0; i < 4; i++)
		{

			/// Make a copy of the image for processing
			EasyImage::Oper(EArithmeticLogicOperation_Copy, &m_SrcImg, &m_ImgProcess);

			/// Position the image ROI
			m_SrcROI.SetPlacement((int)(m_X1) - 25 - (150 * (2 - i)), 2, 150, 308);

			/// Position the mask ROI
			m_MaskROI.SetPlacement((int)(m_X1) - 25 - (150 * (2 - i)), 2, 150, 308);

			/// Clear the objects selection
			m_Selection[i].Clear();

			/// Set the thresholding mode for the blob analysis and specify which layers to encode
			m_Encoder.GetGrayscaleSingleThresholdSegmenter().SetMode( EGrayscaleSingleThreshold_Absolute);
			m_Encoder.GetGrayscaleSingleThresholdSegmenter().SetBlackLayerEncoded(true);
			m_Encoder.GetGrayscaleSingleThresholdSegmenter().SetWhiteLayerEncoded(false);

			/// Process each Reagents Separately 
			switch (i)
			{
			/// Reagent 1
			case 0:
				/// Preprocess the image for bubble detection
				m_TempImg.SetSize(&m_ImgProcess);
				EasyImage::WhiteTopHatDisk(&m_ImgProcess, &m_TempImg, 5);
				EasyImage::Median(&m_TempImg, &m_ImgProcess);

				/// Canny edge for bubbles detection
				m_CannyEdge.SetHighThreshold(0.2f);
				m_CannyEdge.SetLowThreshold(0.1f);
				m_CannyEdge.SetThresholdingMode(ECannyThresholdingMode_Relative);
				m_CannyEdge.SetSmoothingScale(1.05f);
				m_CannyEdge.Apply(m_ImgProcess, m_ImgProcess);

				/// Set the absolute threshold value and specify which layers to encode
				m_Encoder.GetGrayscaleSingleThresholdSegmenter().SetAbsoluteThreshold(90);
				m_Encoder.GetGrayscaleSingleThresholdSegmenter().SetBlackLayerEncoded(false);
				m_Encoder.GetGrayscaleSingleThresholdSegmenter().SetWhiteLayerEncoded(true);

				/// Encode the objects with masking
				m_Encoder.Encode(m_SrcROI, m_MaskROI, m_CodedImage[i]);
				m_Selection[i].AddObjects(m_CodedImage[i]);
				m_Selection[i].RemoveUsingUnsignedIntegerFeature(EFeature_Area, 20, ESingleThresholdMode_LessEqual);
				break;

			/// Reagent 2
			case 1:
				/// Set the absolute threshold value
				m_Encoder.GetGrayscaleSingleThresholdSegmenter().SetAbsoluteThreshold(80);
				/// Encode objects with masking
				m_Encoder.Encode(m_SrcROI, m_MaskROI, m_CodedImage[i]);
				m_Selection[i].AddObjects(m_CodedImage[i]);
				m_Selection[i].RemoveUsingUnsignedIntegerFeature(EFeature_Area, 50, ESingleThresholdMode_LessEqual);
				break;

			/// Reagent 3
			case 2:
				/// Set the absolute threshold value
				m_Encoder.GetGrayscaleSingleThresholdSegmenter().SetAbsoluteThreshold(80);
				/// Encode objects with masking
				m_Encoder.Encode(m_SrcROI, m_MaskROI, m_CodedImage[i]);
				m_Selection[i].AddObjects(m_CodedImage[i]);
				break;

			/// Reagent 4
			case 3:
				/// Set the absolute threshold value
				m_Encoder.GetGrayscaleSingleThresholdSegmenter().SetAbsoluteThreshold(50);
				/// Encode objects with masking
				m_Encoder.Encode(m_SrcROI, m_MaskROI, m_CodedImage[i]);
				m_Selection[i].AddObjects(m_CodedImage[i]);
				m_Selection[i].RemoveUsingUnsignedIntegerFeature(EFeature_Area, 100, ESingleThresholdMode_LessEqual);
				break;

			}

		}

 	}
	catch(EException exc)
	{
		::MessageBox(NULL, exc.What().c_str(), "Error", MB_OK | MB_ICONERROR);
	}


}


void ImageProcessing::Draw(HDC _mContext)
{

	/// Specify the color for drawing
	 ERGBColor redColor(255, 0, 0);
	 ERGBColor blueColor(0, 0, 255);

	try
	{
		/// Exit if no image has been loaded
		if (m_SrcImg.IsVoid())
			return;

		/// Draw the source image
		m_SrcImg.Draw(_mContext);


		/// Draw the detected objects for each reagents 
		for (int i = 0; i < 4; i++)
		{
			if (m_Selection[i].GetElementCount() > 0)
			{

				/// Draw the detected objects
				m_CodedImage[i].Draw(_mContext,redColor, m_Selection[i]);
        
				/// Draw the objects gravity center
				m_CodedImage[i].DrawFeature(_mContext, blueColor, EDrawableFeature_GravityCenter, m_Selection[i]);

			}
		}
		
 	}
	catch(EException exc)
	{
		::MessageBox(NULL, exc.What().c_str(), "Error", MB_OK | MB_ICONERROR);
	}


}
